﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Windows.Devices.SerialCommunication;
using Windows.Devices.Enumeration;
using Windows.Storage.Streams;


namespace UART_GPS
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {

        SerialDevice myUART1;
        DataReader myDataReader;
        private int count = 0;

        public MainWindow()
        {
            InitializeComponent();
            UartSetup();
        }


        public async void UartSetup()
        {
            string aqs = SerialDevice.GetDeviceSelector("UART1");
            DeviceInformationCollection deviceInfo = await DeviceInformation.FindAllAsync(aqs);
            myUART1 = await SerialDevice.FromIdAsync(deviceInfo[0].Id);
            if (myUART1 == null)
            {
                txtStatus.Content = "Error - Serial Port not found";
            }
            else
            {
                txtStatus.Content = "UART1 Found";
                myUART1.BaudRate = 9600;
                myUART1.DataBits = 8;
                myUART1.Parity = SerialParity.None;
                myUART1.StopBits = SerialStopBitCount.One;
                myUART1.Handshake = SerialHandshake.None;
                myUART1.ReadTimeout = TimeSpan.FromMilliseconds(100);
                myUART1.WriteTimeout = TimeSpan.FromMilliseconds(100);

                UartListener();
            }
        }

        private async void UartListener()
        {
            try
            {
                myDataReader = new DataReader(myUART1.InputStream);
                txtStatus.Content = "Awaiting messages";
                while (true)
                {
                    await UartReader();
                }
            }
            catch (TaskCanceledException exTCE)
            {
                txtStatus.Content = "Receive Error " + exTCE.Message;
            }
            catch (Exception ex)
            {
                txtStatus.Content = "Error!: " + ex.ToString();
            }
            finally
            {
                if (myDataReader != null)
                {
                    myDataReader.DetachStream();
                    myDataReader = null;
                }
            }
        }


        private async Task UartReader()
        {
            Task<UInt32> loadAsyncTask;
            uint ReadBufferLength = 1024;
            string GPSOutput;
            myDataReader.InputStreamOptions = InputStreamOptions.Partial;

            loadAsyncTask = myDataReader.LoadAsync(ReadBufferLength).AsTask();
            UInt32 bytesRead = await loadAsyncTask;
            if (bytesRead > 0)
            {

                GPSOutput = myDataReader.ReadString(bytesRead) + "\n";
                txtGPSOutput.Text = GPSOutput.ToString();
                txtStatus.Content = bytesRead.ToString() + " bytes read successfully!";
                GPSParser(GPSOutput);
                count++;
                txtCount.Text = count.ToString();
            }
        }

        private void GPSParser(string GPS)
        {
            string[] gpsValues = GPS.Split(',');
            if (gpsValues.Length != 0)
            {
                if (gpsValues[0].Equals("$GNGGA") && (gpsValues.Length > 1))
                {
                    txtTime.Text = gpsValues[1].ToString();
                    txtLatitude.Text = gpsValues[2].ToString();
                    txtNSLatitude.Text = gpsValues[3].ToString();
                    txtLongitude.Text = gpsValues[4].ToString();
                    txtEWLongitude.Text = gpsValues[5].ToString();
                    txtSats.Text = gpsValues[7].ToString();
                }
            }
        }

    }
}
