using System;
using Windows.Devices.Enumeration;
using Windows.Devices.Spi;


namespace Annabooks.MCP3002
{
    public class MCP3002
    {

        private SpiDevice adcMCP3002;
        private byte[] Channel0 = { 0x68, 0x00 };
        private byte[] Channel1 = { 0x78, 0x00 };
        private byte[] DiffCh0PCh1N = { 0x48, 0x0 };
        private byte[] DiffCh0NCh1P = { 0x58, 0x0 };

        private bool MPC3002Ready = false;
        public string connectionError = "";

#pragma warning disable CS8618 // Non-nullable field must contain a non-null value when exiting constructor. Consider declaring as nullable.
        public MCP3002(string SPI_Port, int frequency, int SPI_Chip_Select, bool SPI_Bus_Shared)
#pragma warning restore CS8618 // Non-nullable field must contain a non-null value when exiting constructor. Consider declaring as nullable.
        {
            MPC3002SPISetup(SPI_Port, frequency, SPI_Chip_Select, SPI_Bus_Shared);
        }

        private async void MPC3002SPISetup(string SPI_Port, int frequency, int SPI_Chip_Select, bool SPI_Bus_Shared)
        {
            try
            {
                SpiConnectionSettings spiSettings = new SpiConnectionSettings(SPI_Chip_Select);
                spiSettings.ClockFrequency = frequency;
                if (SPI_Bus_Shared)
                {
                    spiSettings.SharingMode = SpiSharingMode.Shared;
                }
                else
                {
                    spiSettings.SharingMode = SpiSharingMode.Exclusive;
                }
                spiSettings.Mode = SpiMode.Mode0;

                string aqs = SpiDevice.GetDeviceSelector(SPI_Port);
                DeviceInformationCollection spiDeviceInfo = await DeviceInformation.FindAllAsync(aqs);
                adcMCP3002 = await SpiDevice.FromIdAsync(spiDeviceInfo[0].Id, spiSettings);
            }
            catch (Exception ex)
            {
                connectionError = ex.Message;
            }
            if (adcMCP3002 != null)
            {
                MPC3002Ready = true;
            }
        }

        public int ReadChannel0()
        {
            if (MPC3002Ready == true)
            {
                byte[] adcCh0Out = new byte[2];
                adcMCP3002.TransferFullDuplex(Channel0, adcCh0Out);
                var adcCh0OutInt = (adcCh0Out[0] << 8 | adcCh0Out[1]) & 0x3FF;
                return adcCh0OutInt;
            }
            else
            {
                return 0;
            }
        }

        public int ReadChannel1()
        {
            if (MPC3002Ready == true)
            {
                byte[] adcCh1Out = new byte[2];
                adcMCP3002.TransferFullDuplex(Channel1, adcCh1Out);
                var adcCh1OutInt = (adcCh1Out[0] << 8 | adcCh1Out[1]) & 0x3FF;
                return adcCh1OutInt;
            }
            else
            {
                return 0;
            }
        }

        public int ReadDiff()
        {
            if (MPC3002Ready == false)
            {
                return 0;
            }
            byte[] adcDiffOut = new byte[2];
            adcMCP3002.TransferFullDuplex(DiffCh0PCh1N, adcDiffOut);

            var adcDiffOutInt = (adcDiffOut[0] << 8 | adcDiffOut[1]) & 0x3ff;

            if (adcDiffOutInt > 0) //The output data doesn't have a sign bit. If IN+ is equal to or below IN- the result is 0x00 so we test for 0
            {
                return adcDiffOutInt;
            }
            else //If 0 then we flip the differential input to test for negative - not a real-time solution
            {
                adcMCP3002.TransferFullDuplex(DiffCh0NCh1P, adcDiffOut);
                adcDiffOutInt = (adcDiffOut[0] << 8 | adcDiffOut[1]) & 0x3ff;
                return -adcDiffOutInt;
            }
        }

        public bool MCP3002Status()
        {
            return MPC3002Ready;
        }
    }
}
